@@ -31,20 +31,40 @@ using namespace std;
 static string const database_option("database");
 static string const branch_option("branch");
 static string const key_option("key");
+static string const keydir_option("keydir");
 
 app_state::app_state() 
-  : branch_name(""), db(system_path()), stdhooks(true), rcfiles(true), diffs(false),
+  : branch_name(""), db(system_path()), keys(this), stdhooks(true),
+    rcfiles(true), diffs(false),
     no_merges(false), set_default(false), verbose(false), search_root("/"),
     depth(-1), last(-1), diff_format(unified_diff), diff_args_provided(false),
-    use_lca(false), execute(false)
+    use_lca(false), execute(false), bind_address(""), bind_port(""), 
+    missing(false), unknown(false),
+    confdir(get_default_confdir()), have_set_key_dir(false)
 {
   db.set_app(this);
+  lua.set_app(this);
+  keys.set_key_dir(confdir / "keys");
 }
 
 app_state::~app_state()
 {
 }
 
+void
+app_state::set_is_explicit_option (int option_id)
+{
+  explicit_option_map[option_id] = true;
+}
+
+bool
+app_state::is_explicit_option(int option_id) const
+{
+  std::map<int, bool>::const_iterator i = explicit_option_map.find(option_id);
+  if (i == explicit_option_map.end()) return false;
+  return i->second;
+}
+
 void
 app_state::allow_working_copy()
 {
@@ -252,6 +272,18 @@ app_state::set_database(system_path const & filename)
   options[database_option] = filename.as_internal();
 }
 
+void 
+app_state::set_key_dir(system_path const & filename)
+{
+  if (!filename.empty())
+    {
+      keys.set_key_dir(filename);
+      have_set_key_dir = true;
+    }
+
+  options[keydir_option] = filename.as_internal();
+}
+
 void 
 app_state::set_branch(utf8 const & branch)
 {
@@ -280,6 +312,14 @@ app_state::set_signing_key(utf8 const & key)
   options[key_option] = key;
 }
 
+void 
+app_state::add_key_to_push(utf8 const & key)
+{
+  rsa_keypair_id k;
+  internalize_rsa_keypair_id(key, k);
+  keys_to_push.push_back(k);
+}
+
 void 
 app_state::set_root(system_path const & path)
 {
@@ -385,6 +425,20 @@ app_state::add_rcfile(utf8 const & filename)
   extra_rcfiles.push_back(filename);
 }
 
+void
+app_state::set_confdir(system_path const & cd)
+{
+  confdir = cd;
+  if (!have_set_key_dir)
+    keys.set_key_dir(cd / "keys");
+}
+
+system_path
+app_state::get_confdir()
+{
+  return confdir;
+}
+
 // rc files are loaded after we've changed to the working copy directory so
 // that MT/monotonerc can be loaded between ~/.monotone/monotonerc and other
 // rcfiles
