@@ -156,6 +156,14 @@ void PK_Signer::update(const byte in[], u32bit length)
    emsa->update(in, length);
    }
 
+/*************************************************
+* Add more to the message to be signed           *
+*************************************************/
+void PK_Signer::update(byte in)
+   {
+   update(&in, 1);
+   }
+
 /*************************************************
 * Add more to the message to be signed           *
 *************************************************/
@@ -248,17 +256,17 @@ void PK_Verifier::update(const byte in[], u32bit length)
 /*************************************************
 * Append to the message                          *
 *************************************************/
-void PK_Verifier::update(const MemoryRegion<byte>& in)
+void PK_Verifier::update(byte in)
    {
-   emsa->update(in, in.size());
+   update(&in, 1);
    }
 
 /*************************************************
-* Check a signature                              *
+* Append to the message                          *
 *************************************************/
-bool PK_Verifier::valid_signature(const byte sig[], u32bit length)
+void PK_Verifier::update(const MemoryRegion<byte>& in)
    {
-   return check_signature(sig, length);
+   update(in, in.size());
    }
 
 /*************************************************
