@@ -20,17 +20,19 @@ class Randpool : public RandomNumberGenerator
       bool is_seeded() const;
       void clear() throw();
       std::string name() const;
+
       Randpool();
       ~Randpool();
    private:
-      void add_randomness(const byte[], u32bit) throw();
-      void generate(u64bit) throw();
-      void mix_pool() throw();
+      void add_randomness(const byte[], u32bit);
+      void update_buffer();
+      void mix_pool();
+
       const u32bit ITERATIONS_BEFORE_RESEED, POOL_BLOCKS;
       BlockCipher* cipher;
       HashFunction* hash;
-      SecureVector<byte> pool, output;
-      u32bit counter;
+      SecureVector<byte> pool, poolhash, buffer;
+      u32bit entropy, counter;
    };
 
 }
