@@ -53,6 +53,12 @@ int const one_col = 1;
 int const any_rows = -1;
 int const any_cols = -1;
 
+namespace
+{
+  // track all open databases for close_all_databases() handler
+  set<sqlite3*> sql_contexts;
+}
+
 extern "C" {
 // some wrappers to ease migration
   const char *sqlite3_value_text_s(sqlite3_value *v);
@@ -66,7 +72,7 @@ database::database(system_path const & fn) :
   // non-alphabetic ordering of tables in sql source files. we could create
   // a temporary db, write our intended schema into it, and read it back,
   // but this seems like it would be too rude. possibly revisit this issue.
-  schema("1509fd75019aebef5ac3da3a5edf1312393b70e9"),
+  schema("bd86f9a90b5d552f0be1fa9aee847ea0f317778b"),
   __sql(NULL),
   transaction_level(0)
 {}
@@ -77,9 +83,11 @@ database::check_schema()
   string db_schema_id;  
   calculate_schema_id (__sql, db_schema_id);
   N (schema == db_schema_id,
-     F("database schemas do not match: "
-       "wanted %s, got %s. try migrating database") 
-     % schema % db_schema_id);
+     F("layout of database %s doesn't match this version of monotone\n"
+       "wanted schema %s, got %s\n"
+       "try 'monotone db migrate' to upgrade\n"
+       "(this is irreversible; you may want to make a backup copy first)")
+     % filename % schema % db_schema_id);
 }
 
 // sqlite3_value_text gives a const unsigned char * but most of the time
@@ -316,14 +324,16 @@ dump_index_cb(void *data, int n, char **vals, char **cols)
 void 
 database::dump(ostream & out)
 {
+  transaction_guard guard(*this);
   dump_request req;
   req.out = &out;
   req.sql = sql();
-  out << "BEGIN TRANSACTION;\n";
+  out << "BEGIN EXCLUSIVE;\n";
   int res;
   res = sqlite3_exec(req.sql,
                         "SELECT name, type, sql FROM sqlite_master "
                         "WHERE type='table' AND sql NOT NULL "
+                        "AND name not like 'sqlite_stat%' "
                         "ORDER BY name",
                         dump_table_cb, &req, NULL);
   assert_sqlite3_ok(req.sql);
@@ -334,6 +344,7 @@ database::dump(ostream & out)
                         dump_index_cb, &req, NULL);
   assert_sqlite3_ok(req.sql);
   out << "COMMIT;\n";
+  guard.commit();
 }
 
 void 
@@ -460,7 +471,7 @@ database::version(ostream & out)
 
   calculate_schema_id(__sql, id);
 
-  sqlite3_close(__sql);
+  close();
 
   out << F("database schema version: %s") % id << endl;
 }
@@ -472,8 +483,8 @@ database::migrate()
 
   open();
 
-  migrate_monotone_schema(__sql);
-  sqlite3_close(__sql);
+  migrate_monotone_schema(__sql, __app);
+  close();
 }
 
 void 
@@ -514,22 +525,6 @@ database::rehash()
         ++pubkeys;
       }
   }
-
-  {
-    // rehash all privkeys
-    results res;
-    fetch(res, 2, any_rows, "SELECT id, keydata FROM private_keys");
-    execute("DELETE FROM private_keys");
-    for (size_t i = 0; i < res.size(); ++i)
-      {
-        hexenc<id> tmp;
-        key_hash_code(rsa_keypair_id(res[i][0]), base64< arc4<rsa_priv_key> >(res[i][1]), tmp);
-        execute("INSERT INTO private_keys VALUES(?, ?, ?)", 
-                tmp().c_str(), res[i][0].c_str(), res[i][1].c_str());
-        ++privkeys;
-      }
-  }
-
   guard.commit();
 }
 
@@ -547,16 +542,11 @@ database::~database()
 
   for (map<string, statement>::const_iterator i = statement_cache.begin(); 
        i != statement_cache.end(); ++i)
-    {
-      L(F("%d executions of %s\n") % i->second.count % i->first);
-      sqlite3_finalize(i->second.stmt);
-    }
+    L(F("%d executions of %s\n") % i->second.count % i->first);
+  // trigger destructors to finalize cached statements
+  statement_cache.clear();
 
-  if (__sql)
-    {
-      sqlite3_close(__sql);
-      __sql = 0;
-    }
+  close();
 }
 
 void 
@@ -603,7 +593,7 @@ database::fetch(results & res,
       I(i != statement_cache.end());
 
       const char * tail;
-      sqlite3_prepare(sql(), query, -1, &i->second.stmt, &tail);
+      sqlite3_prepare(sql(), query, -1, i->second.stmt.paddr(), &tail);
       assert_sqlite3_ok(sql());
       L(F("prepared statement %s\n") % query);
 
@@ -612,14 +602,14 @@ database::fetch(results & res,
         F("multiple statements in query: %s\n") % query);
     }
 
-  ncol = sqlite3_column_count(i->second.stmt);
+  ncol = sqlite3_column_count(i->second.stmt());
 
   E(want_cols == any_cols || want_cols == ncol, 
     F("wanted %d columns got %d in query: %s\n") % want_cols % ncol % query);
 
   // bind parameters for this execution
 
-  int params = sqlite3_bind_parameter_count(i->second.stmt);
+  int params = sqlite3_bind_parameter_count(i->second.stmt());
 
   L(F("binding %d parameters for %s\n") % params % query);
 
@@ -637,20 +627,20 @@ database::fetch(results & res,
 
       L(F("binding %d with value '%s'\n") % param % log);
 
-      sqlite3_bind_text(i->second.stmt, param, value, -1, SQLITE_TRANSIENT);
+      sqlite3_bind_text(i->second.stmt(), param, value, -1, SQLITE_TRANSIENT);
       assert_sqlite3_ok(sql());
     }
 
   // execute and process results
 
   nrow = 0;
-  for (rescode = sqlite3_step(i->second.stmt); rescode == SQLITE_ROW; 
-       rescode = sqlite3_step(i->second.stmt))
+  for (rescode = sqlite3_step(i->second.stmt()); rescode == SQLITE_ROW; 
+       rescode = sqlite3_step(i->second.stmt()))
     {
       vector<string> row;
       for (int col = 0; col < ncol; col++) 
         {
-          const char * value = sqlite3_column_text_s(i->second.stmt, col);
+          const char * value = sqlite3_column_text_s(i->second.stmt(), col);
           E(value, F("null result in query: %s\n") % query);
           row.push_back(value);
           //L(F("row %d col %d value='%s'\n") % nrow % col % value);
@@ -661,7 +651,7 @@ database::fetch(results & res,
   if (rescode != SQLITE_DONE)
     assert_sqlite3_ok(sql());
 
-  sqlite3_reset(i->second.stmt);
+  sqlite3_reset(i->second.stmt());
   assert_sqlite3_ok(sql());
 
   nrow = res.size();
@@ -688,7 +678,7 @@ void
 database::begin_transaction() 
 {
   if (transaction_level == 0)
-    execute("BEGIN");
+    execute("BEGIN EXCLUSIVE");
   transaction_level++;
 }
 
@@ -824,6 +814,8 @@ database::put(hexenc<id> const & ident,
   I(ident() != "");
   hexenc<id> tid;
   calculate_ident(dat, tid);
+  MM(ident);
+  MM(tid);
   I(tid == ident);
 
   base64<gzip<data> > dat_packed;
@@ -909,6 +901,30 @@ struct version_cache
 
 static version_cache vcache;
 
+typedef vector< hexenc<id> > version_path;
+
+static void
+extend_path_if_not_cycle(string table_name, 
+                         shared_ptr<version_path> p, 
+                         hexenc<id> const & ext,
+                         set< hexenc<id> > seen_nodes,
+                         vector< shared_ptr<version_path> > & next_paths)
+{
+  for (version_path::const_iterator i = p->begin(); i != p->end(); ++i)
+    {
+      if ((*i)() == ext())
+        throw oops("cycle in table '" + table_name + "', at node " 
+                   + (*i)() + " <- " + ext());
+    }
+
+  if (seen_nodes.find(ext) == seen_nodes.end())
+    {      
+      p->push_back(ext);
+      next_paths.push_back(p);
+      seen_nodes.insert(ext);
+    }
+}
+
 void 
 database::get_version(hexenc<id> const & ident,
                       data & dat,
@@ -948,99 +964,102 @@ database::get_version(hexenc<id> const & ident,
       
       L(F("reconstructing %s in %s\n") % ident % delta_table);
       I(delta_exists(ident, delta_table));
-      
-      // nb: an edge map goes in the direction of the
-      // delta, *not* the direction we discover things in,
-      // i.e. each map is of the form [newid] -> [oldid]
 
-      typedef map< hexenc<id>, hexenc<id> > edgemap;
-      list< shared_ptr<edgemap> > paths;
+      // Our reconstruction algorithm involves keeping a set of parallel
+      // linear paths, starting from ident, moving forward through the
+      // storage DAG until we hit a storage root.
+      //
+      // On each iteration, we extend every active path by one step. If our
+      // extension involves a fork, we duplicate the path. If any path
+      // contains a cycle, we fault. 
+      //
+      // If, by extending a path C, we enter a node which another path
+      // D has already seen, we kill path C. This avoids the possibility of
+      // exponential growth in the number of paths due to extensive forking
+      // and merging.
 
-      set< hexenc<id> > frontier, cycles;
-      frontier.insert(ident);
-
-      bool found_root = false;
-      hexenc<id> root("");
+      vector< shared_ptr<version_path> > live_paths;
 
       string delta_query = "SELECT base FROM " + delta_table + " WHERE id = ?";
 
-      while (! found_root)
+      {
+        shared_ptr<version_path> pth0 = shared_ptr<version_path>(new version_path());      
+        pth0->push_back(ident);
+        live_paths.push_back(pth0);
+      }
+
+      shared_ptr<version_path> selected_path;
+      set< hexenc<id> > seen_nodes;
+
+      while (!selected_path)
         {
-          set< hexenc<id> > next_frontier;
-          shared_ptr<edgemap> frontier_map(new edgemap());
+          vector< shared_ptr<version_path> > next_paths;
 
-          I(!frontier.empty());
-
-          for (set< hexenc<id> >::const_iterator i = frontier.begin();
-               i != frontier.end(); ++i)
+          for (vector<shared_ptr<version_path> >::const_iterator i = live_paths.begin();
+               i != live_paths.end(); ++i)
             {
-              if (vcache.exists(*i) || exists(*i, data_table))
+              shared_ptr<version_path> pth = *i;
+              hexenc<id> tip = pth->back();
+
+              if (vcache.exists(tip) || exists(tip, data_table))
                 {
-                  root = *i;
-                  found_root = true;
+                  selected_path = pth;
                   break;
                 }
               else
                 {
-                  cycles.insert(*i);
-                  results res;
-                  
+                  // This tip is not a root, so extend the path.
+                  results res;                  
                   fetch(res, one_col, any_rows, 
-                        delta_query.c_str(), (*i)().c_str());
+                        delta_query.c_str(), tip().c_str());
 
-                  for (size_t k = 0; k < res.size(); ++k)
+                  I(res.size() != 0);
+
+                  // Replicate the path if there's a fork.
+                  for (size_t k = 1; k < res.size(); ++k)
                     {
-                      hexenc<id> const nxt(res[k][0]);
-
-                      if (cycles.find(nxt) != cycles.end())
-                        throw oops("cycle in table '" + delta_table + "', at node " 
-                                   + (*i)() + " <- " + nxt());
-
-                      next_frontier.insert(nxt);
-
-                      if (frontier_map->find(nxt) == frontier_map->end())
-                        {
-                          L(F("inserting edge: %s <- %s\n") % (*i) % nxt);
-                          frontier_map->insert(make_pair(nxt, *i));
-                        }
-                      else
-                        L(F("skipping merge edge %s <- %s\n") % (*i) % nxt);
+                      shared_ptr<version_path> pthN 
+                        = shared_ptr<version_path>(new version_path(*pth));
+                      extend_path_if_not_cycle(delta_table, pthN, 
+                                               hexenc<id>(res[k][0]),
+                                               seen_nodes, next_paths);
                     }
+
+                  // And extend the base path we're examining.
+                  extend_path_if_not_cycle(delta_table, pth, 
+                                           hexenc<id>(res[0][0]),
+                                           seen_nodes, next_paths);
                 }
             }
-          if (!found_root)
-            {
-              frontier = next_frontier;
-              paths.push_front(frontier_map);
-            }
+
+          live_paths = next_paths;
         }
 
-      // path built, now all we need to do is follow it back
+      // Found a root, now trace it back along the path.
 
-      I(found_root);
-      I(root() != "");
-      data begin;      
+      I(selected_path);
+      I(selected_path->size() > 1);
 
-      if (vcache.exists(root))
+      hexenc<id> curr = selected_path->back();
+      selected_path->pop_back();
+      data begin;
+
+      if (vcache.exists(curr))
         {
-          I(vcache.get(root, begin));
+          I(vcache.get(curr, begin));
         }
       else
         {
-          get(root, begin, data_table);
+          get(curr, begin, data_table);
         }
 
-      hexenc<id> curr = root;
-
       boost::shared_ptr<delta_applicator> app = new_piecewise_applicator();
       app->begin(begin());
       
-      for (list< shared_ptr<edgemap> >::const_iterator p = paths.begin();
-           p != paths.end(); ++p)
+      for (version_path::reverse_iterator i = selected_path->rbegin();
+           i != selected_path->rend(); ++i)
         {
-          shared_ptr<edgemap> i = *p;
-          I(i->find(curr) != i->end());
-          hexenc<id> const nxt = i->find(curr)->second;
+          hexenc<id> const nxt = *i;
 
           if (!vcache.exists(curr))
             {
@@ -1049,7 +1068,6 @@ database::get_version(hexenc<id> const & ident,
               vcache.put(curr, tmp);
             }
 
-
           L(F("following delta %s -> %s\n") % curr % nxt);
           delta del;
           get_delta(nxt, curr, del, delta_table);
@@ -1346,6 +1364,62 @@ database::get_revision(revision_id const & id,
   dat = rdat;
 }
 
+void
+database::deltify_revision(revision_id const & rid)
+{
+  transaction_guard guard(*this);
+  revision_set rev;
+  get_revision(rid, rev);
+  // make sure that all parent revs have their manifests and files
+  // replaced with deltas from this rev's manifest and files
+  // assume that if the manifest is already deltafied, so are the files
+  {
+    MM(rev.new_manifest);
+    for (edge_map::const_iterator i = rev.edges.begin();
+         i != rev.edges.end(); ++i)
+      {
+        manifest_id oldman = edge_old_manifest(i);
+        MM(oldman);
+        if (exists(oldman.inner(), "manifests") &&
+            !(oldman == rev.new_manifest) &&
+            manifest_version_exists(oldman))
+          {
+            manifest_data mdat_new, mdat_old;
+            get_manifest_version(oldman, mdat_old);
+            get_manifest_version(rev.new_manifest, mdat_new);
+            delta delt;
+            diff(mdat_old.inner(), mdat_new.inner(), delt);
+            manifest_delta mdelt(delt);
+            drop(rev.new_manifest.inner(), "manifests");
+            drop(rev.new_manifest.inner(), "manifest_deltas");
+            put_manifest_version(oldman, rev.new_manifest, mdelt);
+          }
+
+        for (change_set::delta_map::const_iterator
+               j = edge_changes(i).deltas.begin();
+             j != edge_changes(i).deltas.end(); ++j)
+          {
+            if (! delta_entry_src(j).inner()().empty() && 
+                  exists(delta_entry_src(j).inner(), "files") &&
+                  file_version_exists(delta_entry_dst(j)))
+              {
+                file_data old_data;
+                file_data new_data;
+                get_file_version(delta_entry_src(j), old_data);
+                get_file_version(delta_entry_dst(j), new_data);
+                delta delt;
+                diff(old_data.inner(), new_data.inner(), delt);
+                file_delta del(delt);
+                drop(delta_entry_dst(j).inner(), "files");
+                drop(delta_entry_dst(j).inner(), "file_deltas");
+                put_file_version(delta_entry_src(j), delta_entry_dst(j), del);
+              }
+          }
+      }
+  }
+  guard.commit();
+}
+
 void 
 database::put_revision(revision_id const & new_id,
                        revision_set const & rev)
@@ -1379,6 +1453,8 @@ database::put_revision(revision_id const & new_id,
               new_id.inner()().c_str());
     }
 
+  deltify_revision(new_id);
+
   check_sane_history(new_id, constants::verify_depth, *__app);
 
   guard.commit();
@@ -1448,11 +1524,9 @@ database::delete_tag_named(cert_value const & tag)
 
 void 
 database::get_key_ids(string const & pattern,
-                      vector<rsa_keypair_id> & pubkeys,
-                      vector<rsa_keypair_id> & privkeys)
+                      vector<rsa_keypair_id> & pubkeys)
 {
   pubkeys.clear();
-  privkeys.clear();
   results res;
 
   if (pattern != "")
@@ -1465,17 +1539,6 @@ database::get_key_ids(string const & pattern,
 
   for (size_t i = 0; i < res.size(); ++i)
     pubkeys.push_back(res[i][0]);
-
-  if (pattern != "")
-    fetch(res, one_col, any_rows, 
-          "SELECT id FROM private_keys WHERE id GLOB ?",
-          pattern.c_str());
-  else
-    fetch(res, one_col, any_rows, 
-          "SELECT id FROM private_keys");
-
-  for (size_t i = 0; i < res.size(); ++i)
-    privkeys.push_back(res[i][0]);
 }
 
 void 
@@ -1489,12 +1552,6 @@ database::get_keys(string const & table, vector<rsa_keypair_id> & keys)
     keys.push_back(res[i][0]);
 }
 
-void 
-database::get_private_keys(vector<rsa_keypair_id> & keys)
-{
-  get_keys("private_keys", keys);
-}
-
 void 
 database::get_public_keys(vector<rsa_keypair_id> & keys)
 {
@@ -1527,25 +1584,6 @@ database::public_key_exists(rsa_keypair_id const & id)
   return false;
 }
 
-bool 
-database::private_key_exists(rsa_keypair_id const & id)
-{
-  results res;
-  fetch(res, one_col, any_rows,
-        "SELECT id FROM private_keys WHERE id = ?",
-        id().c_str());
-  I((res.size() == 1) || (res.size() == 0));
-  if (res.size() == 1)
-    return true;
-  return false;
-}
-
-bool 
-database::key_exists(rsa_keypair_id const & id)
-{
-  return public_key_exists(id) || private_key_exists(id);
-}
-
 void 
 database::get_pubkey(hexenc<id> const & hash, 
                      rsa_keypair_id & id,
@@ -1570,17 +1608,6 @@ database::get_key(rsa_keypair_id const & pub_id,
   pub_encoded = res[0][0];
 }
 
-void 
-database::get_key(rsa_keypair_id const & priv_id, 
-                  base64< arc4<rsa_priv_key> > & priv_encoded)
-{
-  results res;
-  fetch(res, one_col, one_col, 
-        "SELECT keydata FROM private_keys WHERE id = ?", 
-        priv_id().c_str());
-  priv_encoded = res[0][0];
-}
-
 void 
 database::put_key(rsa_keypair_id const & pub_id, 
                   base64<rsa_pub_key> const & pub_encoded)
@@ -1594,36 +1621,6 @@ database::put_key(rsa_keypair_id const & pub_id,
           thash().c_str(), pub_id().c_str(), pub_encoded().c_str());
 }
 
-void 
-database::put_key(rsa_keypair_id const & priv_id, 
-                  base64< arc4<rsa_priv_key> > const & priv_encoded)
-{
-  hexenc<id> thash;
-  key_hash_code(priv_id, priv_encoded, thash);
-  E(!private_key_exists(priv_id),
-    F("another key with name '%s' already exists") % priv_id);
-  execute("INSERT INTO private_keys VALUES(?, ?, ?)", 
-          thash().c_str(), priv_id().c_str(), priv_encoded().c_str());
-}
-
-void 
-database::put_key_pair(rsa_keypair_id const & id, 
-                       base64<rsa_pub_key> const & pub_encoded,
-                       base64< arc4<rsa_priv_key> > const & priv_encoded)
-{
-  transaction_guard guard(*this);
-  put_key(id, pub_encoded);
-  put_key(id, priv_encoded);
-  guard.commit();
-}
-
-void
-database::delete_private_key(rsa_keypair_id const & pub_id)
-{
-  execute("DELETE FROM private_keys WHERE id = ?",
-          pub_id().c_str());
-}
-
 void
 database::delete_public_key(rsa_keypair_id const & pub_id)
 {
@@ -1844,13 +1841,13 @@ database::put_revision_cert(revision<cert> const & cert)
   put_cert(cert.inner(), "revision_certs"); 
 }
 
-void database::get_revision_cert_index(std::vector< std::pair<hexenc<id>,
+void database::get_revision_cert_nobranch_index(std::vector< std::pair<hexenc<id>,
                                        std::pair<revision_id, rsa_keypair_id> > > & idx)
 {
   results res;
   fetch(res, 3, any_rows, 
         "SELECT hash, id, keypair "
-        "FROM 'revision_certs'");
+        "FROM 'revision_certs' WHERE name != 'branch'");
 
   idx.clear();
   idx.reserve(res.size());
@@ -2103,16 +2100,7 @@ database::complete(string const & partial,
         pattern.c_str());
 
   for (size_t i = 0; i < res.size(); ++i)
-    completions.insert(make_pair(key_id(res[i][0]), utf8(res[i][1])));  
-
-  res.clear();
-
-  fetch(res, 2, any_rows,
-        "SELECT hash, id FROM private_keys WHERE hash GLOB ?",
-        pattern.c_str());
-
-  for (size_t i = 0; i < res.size(); ++i)
-    completions.insert(make_pair(key_id(res[i][0]), utf8(res[i][1])));  
+    completions.insert(make_pair(key_id(res[i][0]), utf8(res[i][1])));
 }
 
 using selectors::selector_type;
@@ -2132,6 +2120,10 @@ static void selector_to_certname(selector_type ty,
       prefix = suffix = "";
       s = branch_cert_name;
       break;
+    case selectors::sel_head:
+      prefix = suffix = "";
+      s = branch_cert_name;
+      break;
     case selectors::sel_date:
     case selectors::sel_later:
     case selectors::sel_earlier:
@@ -2154,6 +2146,7 @@ void database::complete(selector_type ty,
                         vector<pair<selector_type, string> > const & limit,
                         set<string> & completions)
 {
+  //L(F("database::complete for partial '%s'\n") % partial);
   completions.clear();
 
   // step 1: the limit is transformed into an SQL select statement which
@@ -2220,27 +2213,88 @@ void database::complete(selector_type ty,
               lim += (boost::format(" AND unbase64(value) glob '*%s*'")
                       % i->second).str();     
             }
+          else if (i->first == selectors::sel_head) 
+            {
+              // get branch names
+              vector<cert_value> branch_names;
+              if (i->second.size() == 0)
+                {
+                  __app->require_working_copy("the empty head selector h: refers to the head of the current branch");
+                  branch_names.push_back((__app->branch_name)());
+                }
+              else
+                {
+                  string subquery = (boost::format("SELECT DISTINCT value FROM revision_certs WHERE name='%s' and unbase64(value) glob '%s'") 
+                                     % branch_cert_name % i->second).str();
+                  results res;
+                  fetch(res, one_col, any_rows, subquery.c_str());
+                  for (size_t i = 0; i < res.size(); ++i)
+                    {
+                      base64<data> row_encoded(res[i][0]);
+                      data row_decoded;
+                      decode_base64(row_encoded, row_decoded);
+                      branch_names.push_back(row_decoded());
+                    }
+                }
+
+              // for each branch name, get the branch heads
+              set<revision_id> heads;
+              for (vector<cert_value>::const_iterator bn = branch_names.begin(); bn != branch_names.end(); bn++)
+                {
+                  set<revision_id> branch_heads;
+                  get_branch_heads(*bn, *__app, branch_heads);
+                  heads.insert(branch_heads.begin(), branch_heads.end());
+                  L(F("after get_branch_heads for %s, heads has %d entries\n") % (*bn) % heads.size());
+                }
+
+              lim += "SELECT id FROM revision_certs WHERE id IN (";
+              if (heads.size())
+                {
+                  set<revision_id>::const_iterator r = heads.begin();
+                  lim += (boost::format("'%s'") % r->inner()()).str();
+                  r++;
+                  while (r != heads.end())
+                    {
+                      lim += (boost::format(", '%s'") % r->inner()()).str();
+                      r++;
+                    }
+                }
+              lim += ") ";
+            }
           else
             {
               string certname;
               string prefix;
               string suffix;
               selector_to_certname(i->first, certname, prefix, suffix);
-              lim += (boost::format("SELECT id FROM revision_certs WHERE name='%s' AND ") % certname).str();
-              switch (i->first)
+              L(F("processing selector type %d with i->second '%s'\n") % ty % i->second);
+              if ((i->first == selectors::sel_branch) && (i->second.size() == 0))
                 {
-                case selectors::sel_earlier:
-                  lim += (boost::format("unbase64(value) <= X'%s'") % encode_hexenc(i->second)).str();
-                  break;
-                case selectors::sel_later:
-                  lim += (boost::format("unbase64(value) > X'%s'") % encode_hexenc(i->second)).str();
-                  break;
-                default:
-                  lim += (boost::format("unbase64(value) glob '%s%s%s'")
-                          % prefix % i->second % suffix).str();
-                  break;
+                  __app->require_working_copy("the empty branch selector b: refers to the current branch");
+                  // FIXME: why do we have to glob on the unbase64(value), rather than being able to use == ?
+                  lim += (boost::format("SELECT id FROM revision_certs WHERE name='%s' AND unbase64(value) glob '%s'")
+                          % branch_cert_name % __app->branch_name).str();
+                  L(F("limiting to current branch '%s'\n") % __app->branch_name);
+                }
+              else
+                {
+                  lim += (boost::format("SELECT id FROM revision_certs WHERE name='%s' AND ") % certname).str();
+                  switch (i->first)
+                    {
+                    case selectors::sel_earlier:
+                      lim += (boost::format("unbase64(value) <= X'%s'") % encode_hexenc(i->second)).str();
+                      break;
+                    case selectors::sel_later:
+                      lim += (boost::format("unbase64(value) > X'%s'") % encode_hexenc(i->second)).str();
+                      break;
+                    default:
+                      lim += (boost::format("unbase64(value) glob '%s%s%s'")
+                              % prefix % i->second % suffix).str();
+                      break;
+                    }
                 }
             }
+          //L(F("found selector type %d, selecting_head is now %d\n") % i->first % selecting_head);
         }
     }
   lim += ")";
@@ -2286,7 +2340,7 @@ void database::complete(selector_type ty,
   fetch(res, one_col, any_rows, query.c_str());
   for (size_t i = 0; i < res.size(); ++i)
     {
-      if (ty == selectors::sel_ident)
+      if (ty == selectors::sel_ident) 
         completions.insert(res[i][0]);
       else
         {
@@ -2454,17 +2508,44 @@ database::check_filename()
 }
 
 
+bool
+database::database_specified()
+{
+  return !filename.empty();
+}
+
+
 void
 database::open()
 {
   int error;
 
+  I(!__sql);
+
   error = sqlite3_open(filename.as_external().c_str(), &__sql);
 
+  if (__sql)
+    {
+      I(sql_contexts.find(__sql) == sql_contexts.end());
+      sql_contexts.insert(__sql);
+    }
+
   N(!error, (F("could not open database '%s': %s")
              % filename % string(sqlite3_errmsg(__sql))));
 }
 
+void
+database::close()
+{
+  if (__sql)
+    {
+      sqlite3_close(__sql);
+      I(sql_contexts.find(__sql) != sql_contexts.end());
+      sql_contexts.erase(__sql);
+      __sql = 0;
+    }
+}
+
 
 // transaction guards
 
@@ -2485,3 +2566,22 @@ transaction_guard::commit()
 {
   committed = true;
 }
+
+// called to avoid foo.db-journal files hanging around if we exit cleanly
+// without unwinding the stack (happens with SIGINT & SIGTERM)
+void
+close_all_databases()
+{
+  L(F("attempting to rollback and close %d databases") % sql_contexts.size());
+  for (set<sqlite3*>::iterator i = sql_contexts.begin();
+       i != sql_contexts.end(); i++)
+    {
+      // the ROLLBACK is required here, even though the sqlite docs
+      // imply that transactions are rolled back on database closure
+      int exec_err = sqlite3_exec(*i, "ROLLBACK", NULL, NULL, NULL);
+      int close_err = sqlite3_close(*i);
+
+      L(F("exec_err = %d, close_err = %d") % exec_err % close_err);
+    }
+  sql_contexts.clear();
+}
