@@ -13,6 +13,7 @@
 #include "revision.hh"
 #include "ui.hh"
 #include "vocab.hh"
+#include "transforms.hh"
 
 // the database has roughly the following structure
 //
@@ -56,7 +57,12 @@ struct checked_manifest {
   size_t revision_refs; // number of revision references to this manifest
   size_t missing_files; // number of missing files referenced by this manifest
 
-  checked_manifest(): found(false), revision_refs(0), missing_files(0) {}
+  bool parseable;       // read_manifest_map does not throw
+  bool normalized;      // write_manifest_map( read_manifest_map(dat) ) == dat
+
+  checked_manifest(): 
+    found(false), revision_refs(0), 
+    missing_files(0), parseable(false), normalized(false) {}
 };
 
 // the number of times a revision is referenced (revision_refs)
@@ -79,6 +85,9 @@ struct checked_revision {
   
   size_t cert_refs;            // number of references to this revision by revision certs;
 
+  bool parseable;              // read_revision_set does not throw
+  bool normalized;             // write_revision_set( read_revision_set(dat) ) == dat
+
   std::string history_error;
 
   std::set<revision_id> parents;
@@ -88,7 +97,7 @@ struct checked_revision {
     found(false),
     revision_refs(0), ancestry_parent_refs(0), ancestry_child_refs(0), 
     missing_manifests(0), missing_revisions(0), incomplete_manifests(0), 
-    cert_refs(0) {}
+    cert_refs(0), parseable(false), normalized(false) {}
 };
 
 static void
@@ -99,7 +108,7 @@ check_files(app_state & app, std::map<file_id, checked_file> & checked_files)
   app.db.get_file_ids(files);
   L(F("checking %d files\n") % files.size());
 
-  ticker ticks("files", "f", files.size()/70+1);
+  ticker ticks(_("files"), "f", files.size()/70+1);
 
   for (std::set<file_id>::const_iterator i = files.begin();
        i != files.end(); ++i) 
@@ -124,7 +133,7 @@ check_manifests(app_state & app,
   app.db.get_manifest_ids(manifests);
   L(F("checking %d manifests\n") % manifests.size());
 
-  ticker ticks("manifests", "m", manifests.size()/70+1);
+  ticker ticks(_("manifests"), "m", manifests.size()/70+1);
 
   for (std::set<manifest_id>::const_iterator i = manifests.begin();
        i != manifests.end(); ++i) 
@@ -135,7 +144,25 @@ check_manifests(app_state & app,
       checked_manifests[*i].found = true;
 
       manifest_map man;
-      read_manifest_map(data, man);
+      try
+        {
+          read_manifest_map(data, man);
+        }
+      catch (std::logic_error & e)
+        {
+          L(F("error parsing manifest %s: %s") % *i % e.what());
+          checked_manifests[*i].parseable = false;
+          continue;
+        }
+      checked_manifests[*i].parseable = true;
+
+      // normalisation check
+      manifest_id norm_ident;
+      manifest_data norm_data;
+      write_manifest_map(man, norm_data);
+      calculate_ident(norm_data, norm_ident);
+      if (norm_ident == *i)
+          checked_manifests[*i].normalized = true;
 
       for (manifest_map::const_iterator entry = man.begin(); entry != man.end();
            ++entry)
@@ -162,7 +189,7 @@ check_revisions(app_state & app,
   app.db.get_revision_ids(revisions);
   L(F("checking %d revisions\n") % revisions.size());
 
-  ticker ticks("revisions", "r", revisions.size()/70+1);
+  ticker ticks(_("revisions"), "r", revisions.size()/70+1);
 
   for (std::set<revision_id>::const_iterator i = revisions.begin();
        i != revisions.end(); ++i) 
@@ -173,7 +200,25 @@ check_revisions(app_state & app,
       checked_revisions[*i].found = true;
 
       revision_set rev;
-      read_revision_set(data, rev);
+      try
+        {
+          read_revision_set(data, rev);
+        }
+      catch (std::logic_error & e)
+        {
+          L(F("error parsing revision %s: %s") % *i % e.what());
+          checked_revisions[*i].parseable = false;
+          continue;
+        }
+      checked_revisions[*i].parseable = true;
+
+      // normalisation check
+      revision_id norm_ident;
+      revision_data norm_data;
+      write_revision_set(rev, norm_data);
+      calculate_ident(norm_data, norm_ident);
+      if (norm_ident == *i)
+          checked_revisions[*i].normalized = true;
 
       checked_manifests[rev.new_manifest].revision_refs++;
 
@@ -243,7 +288,7 @@ check_ancestry(app_state & app,
   app.db.get_revision_ancestry(graph);
   L(F("checking %d ancestry edges\n") % graph.size());
 
-  ticker ticks("ancestry", "a", graph.size()/70+1);
+  ticker ticks(_("ancestry"), "a", graph.size()/70+1);
 
   // checked revision has set of parents
   // graph has revision and associated parents
@@ -276,7 +321,7 @@ check_keys(app_state & app,
 
   L(F("checking %d public keys\n") % pubkeys.size());
 
-  ticker ticks("keys", "k", 1);
+  ticker ticks(_("keys"), "k", 1);
 
   for (std::vector<rsa_keypair_id>::const_iterator i = pubkeys.begin();
        i != pubkeys.end(); ++i)
@@ -302,7 +347,7 @@ check_certs(app_state & app,
 
   L(F("checking %d revision certs\n") % certs.size());
 
-  ticker ticks("certs", "c", certs.size()/70+1);
+  ticker ticks(_("certs"), "c", certs.size()/70+1);
 
   for (std::vector< revision<cert> >::const_iterator i = certs.begin();
        i != certs.end(); ++i)
@@ -314,7 +359,7 @@ check_certs(app_state & app,
         {
           std::string signed_text;
           cert_signable_text(i->inner(), signed_text);
-          checked.good_sig = check_signature(app.lua, i->inner().key, 
+          checked.good_sig = check_signature(app, i->inner().key, 
                                              checked_keys[i->inner().key].pub_encoded, 
                                              signed_text, i->inner().sig);
         }
@@ -332,7 +377,7 @@ check_sane(app_state & app,
 {
   L(F("checking local history of %d revisions\n") % checked_revisions.size());
 
-  ticker ticks("revisions", "r", 1);
+  ticker ticks(_("revisions"), "r", 1);
 
   for (std::map<revision_id, checked_revision>::iterator 
          i = checked_revisions.begin(); i != checked_revisions.end(); ++i)
@@ -382,7 +427,9 @@ static void
 report_manifests(std::map<manifest_id, checked_manifest> const & checked_manifests, 
                  size_t & missing_manifests, 
                  size_t & unreferenced_manifests,
-                 size_t & incomplete_manifests)
+                 size_t & incomplete_manifests,
+                 size_t & non_parseable_manifests,
+                 size_t & non_normalized_manifests)
 {
   for (std::map<manifest_id, checked_manifest>::const_iterator 
          i = checked_manifests.begin(); i != checked_manifests.end(); ++i)
@@ -408,6 +455,20 @@ report_manifests(std::map<manifest_id, checked_manifest> const & checked_manifes
           P(F("manifest %s incomplete (%d missing files)\n") 
             % i->first % manifest.missing_files);
         }
+
+      if (!manifest.parseable)
+        {
+          non_parseable_manifests++;
+          P(F("manifest %s is not parseable (perhaps with unnormalized paths?)\n")
+            % i->first);
+        }
+
+      if (manifest.parseable && !manifest.normalized)
+        {
+          non_normalized_manifests++;
+          P(F("manifest %s is not in normalized form\n")
+            % i->first);
+        }
     }
 }
 
@@ -417,7 +478,9 @@ report_revisions(std::map<revision_id, checked_revision> const & checked_revisio
                  size_t & incomplete_revisions,
                  size_t & mismatched_parents,
                  size_t & mismatched_children,
-                 size_t & bad_history)
+                 size_t & bad_history,
+                 size_t & non_parseable_revisions,
+                 size_t & non_normalized_revisions)
 {
   for (std::map<revision_id, checked_revision>::const_iterator 
          i = checked_revisions.begin(); i != checked_revisions.end(); ++i)
@@ -473,9 +536,25 @@ report_revisions(std::map<revision_id, checked_revision> const & checked_revisio
       if (!revision.history_error.empty())
         {
           bad_history++;
+          std::string tmp = revision.history_error;
+          if (tmp[tmp.length() - 1] == '\n')
+            tmp.erase(tmp.length() - 1);
           P(F("revision %s has bad history (%s)\n")
-            % i->first
-            % revision.history_error); 
+            % i->first % tmp);
+        }
+
+      if (!revision.parseable)
+        {
+          non_parseable_revisions++;
+          P(F("revision %s is not parseable (perhaps with unnormalized paths?)\n")
+            % i->first);
+        }
+
+      if (revision.parseable && !revision.normalized)
+        {
+          non_normalized_revisions++;
+          P(F("revision %s is not in normalized form\n")
+            % i->first);
         }
     }
 }
@@ -587,12 +666,16 @@ check_db(app_state & app)
   size_t missing_manifests = 0;
   size_t unreferenced_manifests = 0;
   size_t incomplete_manifests = 0;
+  size_t non_parseable_manifests = 0;
+  size_t non_normalized_manifests = 0;
 
   size_t missing_revisions = 0;
   size_t incomplete_revisions = 0;
   size_t mismatched_parents = 0;
   size_t mismatched_children = 0;
   size_t bad_history = 0;
+  size_t non_parseable_revisions = 0;
+  size_t non_normalized_revisions = 0;
   
   size_t missing_keys = 0;
 
@@ -613,12 +696,16 @@ check_db(app_state & app)
   report_files(checked_files, missing_files, unreferenced_files);
 
   report_manifests(checked_manifests, 
-                   missing_manifests, unreferenced_manifests, incomplete_manifests);
+                   missing_manifests, unreferenced_manifests, 
+                   incomplete_manifests,
+                   non_parseable_manifests,
+                   non_normalized_manifests);
 
   report_revisions(checked_revisions,
                    missing_revisions, incomplete_revisions, 
                    mismatched_parents, mismatched_children,
-                   bad_history);
+                   bad_history, non_parseable_revisions,
+                   non_normalized_revisions);
 
   report_keys(checked_keys, missing_keys);
 
@@ -637,6 +724,11 @@ check_db(app_state & app)
     W(F("%d unreferenced manifests\n") % unreferenced_manifests);
   if (incomplete_manifests > 0)
     W(F("%d incomplete manifests\n") % incomplete_manifests);
+  if (non_parseable_manifests > 0)
+    W(F("%d manifests not parseable (perhaps with invalid paths)\n")
+      % non_parseable_manifests);
+  if (non_normalized_manifests > 0)
+    W(F("%d manifests not in normalized form\n") % non_normalized_manifests);
 
   if (missing_revisions > 0)
     W(F("%d missing revisions\n") % missing_revisions);
@@ -648,6 +740,11 @@ check_db(app_state & app)
     W(F("%d mismatched children\n") % mismatched_children);
   if (bad_history > 0)
     W(F("%d revisions with bad history\n") % bad_history);
+  if (non_parseable_revisions > 0)
+    W(F("%d revisions not parseable (perhaps with invalid paths)\n")
+      % non_parseable_revisions);
+  if (non_normalized_revisions > 0)
+    W(F("%d revisions not in normalized form\n") % non_normalized_revisions);
 
   if (missing_keys > 0)
     W(F("%d missing keys\n") % missing_keys);
@@ -663,27 +760,38 @@ check_db(app_state & app)
 
   size_t total = missing_files + unreferenced_files +
     missing_manifests + unreferenced_manifests + incomplete_manifests +
+    non_parseable_manifests + non_normalized_manifests +
     missing_revisions + incomplete_revisions + 
+    non_parseable_revisions + non_normalized_revisions +
     mismatched_parents + mismatched_children +
     bad_history +
     missing_certs + mismatched_certs +
     unchecked_sigs + bad_sigs +
     missing_keys;
+  // unreferenced files and manifests and mismatched certs are not actually
+  // serious errors; odd, but nothing will break.
+  size_t serious = missing_files + 
+    missing_manifests + incomplete_manifests +
+    non_parseable_manifests + non_normalized_manifests +
+    missing_revisions + incomplete_revisions + 
+    non_parseable_revisions + non_normalized_revisions +
+    mismatched_parents + mismatched_children +
+    bad_history +
+    missing_certs +
+    unchecked_sigs + bad_sigs +
+    missing_keys;
 
-  if (total > 0)
-    P(F("check complete: %d files; %d manifests; %d revisions; %d keys; %d certs; %d problems detected\n") 
-      % checked_files.size()
-      % checked_manifests.size()
-      % checked_revisions.size()
-      % checked_keys.size()
-      % total_certs
-      % total);
+  P(F("check complete: %d files; %d manifests; %d revisions; %d keys; %d certs\n")
+    % checked_files.size()
+    % checked_manifests.size()
+    % checked_revisions.size()
+    % checked_keys.size()
+    % total_certs);
+  P(F("total problems detected: %d (%d serious)\n") % total % serious);
+  if (serious)
+    E(false, F("serious problems detected"));
+  else if (total)
+    P(F("minor problems detected\n"));
   else
-    P(F("check complete: %d files; %d manifests; %d revisions; %d keys; %d certs; database is good\n") 
-      % checked_files.size()
-      % checked_manifests.size()
-      % checked_revisions.size()
-      % checked_keys.size()
-      % total_certs
-      );
+    P(F("database is good\n"));
 }
