@@ -49,11 +49,11 @@ struct work_vec
 {
   long lo;
   long hi;
-  static std::vector<long> vec;
+  static std::vector<long, QA(long)> vec;
   work_vec(long lo, long hi) : 
     lo(lo), hi(hi)
   {
-    I(hi >= lo);
+    // I(hi >= lo);
     size_t len = (hi - lo) + 1;
     vec.resize(len);
     vec.assign(len, -1);
@@ -61,21 +61,21 @@ struct work_vec
 
   inline long & operator[](long t) 
   {
-    I(t >= lo && t <= hi);
+    // I(t >= lo && t <= hi);
     return vec[t-lo]; 
   }
 };
 
-std::vector<long> work_vec::vec;
+std::vector<long, QA(long)> work_vec::vec;
 
 template <typename A,
-	  typename B,
-	  typename LCS>	
+          typename B,
+          typename LCS>        
 struct jaffer_edit_calculator
 {
 
-  typedef std::vector<long> cost_vec;
-  typedef std::vector<long> edit_vec;
+  typedef std::vector<long, QA(long)> cost_vec;
+  typedef std::vector<long, QA(long)> edit_vec;
 
   template <typename T>
   struct subarray
@@ -89,32 +89,32 @@ struct jaffer_edit_calculator
     subarray(T b, long s, long e) : 
       base(b), start(s), end(e) {}
 
-    long size() const
+    inline long size() const
     {
       if (end < start)
-	return start - end;
+        return start - end;
       else
-	return end - start;
+        return end - start;
     }
     
-    subarray subset(long s, long e) const
+    inline subarray subset(long s, long e) const
     {
       return subarray(base + std::min(start, end), s, e);
     }
     
-    vt const & operator[](size_t idx) const
+    inline vt const & operator[](size_t idx) const
     {
       if (end < start)
-	  return *(base + (start - (idx + 1)));
+          return *(base + (start - (idx + 1)));
       else
-	  return *(base + (start + idx));
+          return *(base + (start + idx));
     }
   };    
 
   static long run(work_vec & fp, long k, 
-		  subarray<A> const & a, long m,
-		  subarray<B> const & b, long n,
-		  cost_vec & CC, long p)
+                  subarray<A> const & a, long m,
+                  subarray<B> const & b, long n,
+                  cost_vec & CC, long p)
   {
     long cost = k + 2*p;
     
@@ -126,19 +126,19 @@ struct jaffer_edit_calculator
     I(x >= 0);
 
     while (true)
-      {	
-	// record costs along the way
-	long xcst = m - x;
-	if (y < static_cast<long>(CC.size()) && xcst >= 0)
-	  {
-	    CC[y] = std::min(xcst + cost, CC[y]); 
-	  }	
-	if (x < m && y < n && a[x] == b[y])
-	  {
-	    ++x; ++y;
-	  }
-	else
-	  break;
+      {        
+        // record costs along the way
+        long xcst = m - x;
+        if (y < static_cast<long>(CC.size()) && xcst >= 0)
+          {
+            CC[y] = std::min(xcst + cost, CC[y]); 
+          }        
+        if (x < m && y < n && a[x] == b[y])
+          {
+            ++x; ++y;
+          }
+        else
+          break;
       }
 
     fp[k] = y;
@@ -147,16 +147,16 @@ struct jaffer_edit_calculator
 
   // 'compare' here is the core myers, manber and miller algorithm.
   static long compare(cost_vec & costs, 
-		      subarray<A> const & a, long m,
-		      subarray<B> const & b, long n,
-		      long p_lim,
-		      bool full_scan = true)
+                      subarray<A> const & a, long m,
+                      subarray<B> const & b, long n,
+                      long p_lim,
+                      bool full_scan = true)
   {
     long lo = -(m+1), hi = (1+n);
     if (full_scan)
       {
-	lo = -(p_lim + 1);
-	hi = p_lim + 1 + (n-m);
+        lo = -(p_lim + 1);
+        hi = p_lim + 1 + (n-m);
       }
     work_vec fp(lo, hi);
 
@@ -166,31 +166,31 @@ struct jaffer_edit_calculator
     for (; p <= p_lim; ++p)
       {
 
-	// lower sweep
-	for (long k = -p; k < delta; ++k)
-	  run(fp, k, a, m, b, n, costs, p);
+        // lower sweep
+        for (long k = -p; k < delta; ++k)
+          run(fp, k, a, m, b, n, costs, p);
 
-	// upper sweep
-	for (long k = delta + p; k > delta; --k)
-	  run(fp, k, a, m, b, n, costs, p);
+        // upper sweep
+        for (long k = delta + p; k > delta; --k)
+          run(fp, k, a, m, b, n, costs, p);
 
-	// middle
-	long fpval = run(fp, delta, a, m, b, n, costs, p);
+        // middle
+        long fpval = run(fp, delta, a, m, b, n, costs, p);
 
-	// we can bail early if not doing a full scan
-	if (!full_scan && n <= fpval)
-	  break;
+        // we can bail early if not doing a full scan
+        if (!full_scan && n <= fpval)
+          break;
       }
 
     return delta + 2*p;
   }
   
   static long divide_and_conquer(subarray<A> const & a, long start_a, long end_a,
-				 subarray<B> const & b, long start_b, long end_b,
-				 edit_vec & edits,
-				 unsigned long edx, 
-				 long polarity,
-				 long p_lim)
+                                 subarray<B> const & b, long start_b, long end_b,
+                                 edit_vec & edits,
+                                 unsigned long edx, 
+                                 long polarity,
+                                 long p_lim)
   {
     long mid_a = (start_a + end_a) / 2;
     long len_b = end_b - start_b;
@@ -210,12 +210,12 @@ struct jaffer_edit_calculator
     cost_vec rr(len_b + 1, len_a + len_b);
 
     compare (cc, 
-	     a.subset(start_a, mid_a), (mid_a - start_a),
-	     b.subset(start_b, end_b), len_b, std::min(p_lim, len_a));
+             a.subset(start_a, mid_a), (mid_a - start_a),
+             b.subset(start_b, end_b), len_b, std::min(p_lim, len_a));
 
     compare (rr, 
-	     a.subset(end_a, mid_a), (end_a - mid_a),
-	     b.subset(end_b, start_b), len_b, std::min(p_lim, len_a));
+             a.subset(end_a, mid_a), (end_a - mid_a),
+             b.subset(end_b, start_b), len_b, std::min(p_lim, len_a));
 
     long b_split = mid_split(len_a, len_b, rr, cc, tcst);
 
@@ -223,16 +223,16 @@ struct jaffer_edit_calculator
     long est_r = rr[len_b - b_split];
 
     long cost_c = diff_to_et (a, start_a, mid_a,
-			      b, start_b, start_b + b_split,
-			      edits, edx, polarity, 
-			      (est_c - (b_split - (mid_a - start_a))) / 2);
+                              b, start_b, start_b + b_split,
+                              edits, edx, polarity, 
+                              (est_c - (b_split - (mid_a - start_a))) / 2);
 
     I(cost_c == est_c);
     
     long cost_r = diff_to_et (a, mid_a, end_a,
-			      b, start_b + b_split, end_b,
-			      edits, est_c + edx, polarity,
-			      (est_r - ((len_b - b_split) - (end_a - mid_a))) / 2);
+                              b, start_b + b_split, end_b,
+                              edits, est_c + edx, polarity,
+                              (est_r - ((len_b - b_split) - (end_a - mid_a))) / 2);
 
     I(cost_r == est_r);
     
@@ -240,29 +240,29 @@ struct jaffer_edit_calculator
   }
   
   static long mid_split(long m, long n, 
-			cost_vec const & rr,
-			cost_vec const & cc,
-			long cost)
+                        cost_vec const & rr,
+                        cost_vec const & cc,
+                        long cost)
   {
     long cdx = 1 + n/2;
     long rdx = n/2;
     while (true)
       {
-	I (rdx >= 0);
+        I (rdx >= 0);
 
-	if (cost == (cc[rdx] + rr[n-rdx])) 
-	    return rdx;
-	if (cost == (cc[cdx] + rr[n-cdx]))
-	    return cdx;	  
-	--rdx;
-	++cdx;
+        if (cost == (cc[rdx] + rr[n-rdx])) 
+            return rdx;
+        if (cost == (cc[cdx] + rr[n-cdx]))
+            return cdx;          
+        --rdx;
+        ++cdx;
       }
   }
   
   
   static void order_edits(edit_vec const & edits,
-			  long sign,
-			  edit_vec & nedits)
+                          long sign,
+                          edit_vec & nedits)
   {
     nedits.clear();
     nedits.resize(edits.size());
@@ -270,8 +270,8 @@ struct jaffer_edit_calculator
     
     if (cost == 0)
       {
-	nedits = edits;
-	return;
+        nedits = edits;
+        return;
       }
     
     edit_vec sedits = edits;
@@ -291,40 +291,40 @@ struct jaffer_edit_calculator
     while (bdx < len_b || adx < len_a)
       {
 
-	long del = (ddx < 0) ? 0 : sedits[ddx];
-	long ins = (idx >= cost) ? 0 : sedits[idx];
-	
-	if (del < 0 && adx >= (-1 - del) &&
-	    ins > 0 && bdx >= (-1 + ins))
-	  {
-	    nedits[ndx] = del;
-	    nedits[ndx+1] = ins;
-	    --ddx; ++idx; ndx += 2; ++adx; ++bdx;	      
-	  }
-	else if (del < 0 && adx >= (-1 - del))
-	  {
-	    nedits[ndx] = del;
-	    --ddx; ++ndx; ++adx;
-	  }
-	else if (ins > 0 && bdx >= (-1 + ins))
-	  {
-	    nedits[ndx] = ins;
-	    ++idx; ++ndx; ++bdx;
-	  }
-	else
-	  {
-	    ++adx; ++bdx;
-	  }
+        long del = (ddx < 0) ? 0 : sedits[ddx];
+        long ins = (idx >= cost) ? 0 : sedits[idx];
+        
+        if (del < 0 && adx >= (-1 - del) &&
+            ins > 0 && bdx >= (-1 + ins))
+          {
+            nedits[ndx] = del;
+            nedits[ndx+1] = ins;
+            --ddx; ++idx; ndx += 2; ++adx; ++bdx;              
+          }
+        else if (del < 0 && adx >= (-1 - del))
+          {
+            nedits[ndx] = del;
+            --ddx; ++ndx; ++adx;
+          }
+        else if (ins > 0 && bdx >= (-1 + ins))
+          {
+            nedits[ndx] = ins;
+            ++idx; ++ndx; ++bdx;
+          }
+        else
+          {
+            ++adx; ++bdx;
+          }
       }
   }
   
   // trims and calls diff_to_ez
   static long diff_to_et(subarray<A> const & a, long start_a, long end_a,
-			 subarray<B> const & b, long start_b, long end_b,
-			 std::vector<long> & edits,
-			 long edx,
-			 long polarity,
-			 long p_lim)
+                         subarray<B> const & b, long start_b, long end_b,
+                         std::vector<long, QA(long)> & edits,
+                         long edx,
+                         long polarity,
+                         long p_lim)
   {
 
     I(start_a >= 0);
@@ -341,32 +341,32 @@ struct jaffer_edit_calculator
     long bsx, bdx, asx, adx;
     
     for (bdx = end_b - 1, adx = end_a - 1;
-	 (start_b <= bdx) && (start_a <= adx) && (a[adx] == b[bdx]);	   
-	 --bdx, --adx);
+         (start_b <= bdx) && (start_a <= adx) && (a[adx] == b[bdx]);           
+         --bdx, --adx);
     
     for (bsx = start_b, asx = start_a;
-	 (bsx < bdx) && (asx < adx) && (a[asx] == b[bsx]);
-	 ++bsx, ++asx);
+         (bsx < bdx) && (asx < adx) && (a[asx] == b[bsx]);
+         ++bsx, ++asx);
     
     // we've trimmed; now call diff_to_ez.
     
     long delta = (bdx - bsx) - (adx - asx);
     if (delta < 0)
       return diff_to_ez (b, bsx, bdx+1, 
-			 a, asx, adx+1,
-			 edits, edx, -polarity, delta + p_lim);
+                         a, asx, adx+1,
+                         edits, edx, -polarity, delta + p_lim);
     else
       return diff_to_ez (a, asx, adx+1,
-			 b, bsx, bdx+1, 
-			 edits, edx, polarity, p_lim);	      
+                         b, bsx, bdx+1, 
+                         edits, edx, polarity, p_lim);              
   }
   
   static long diff_to_ez(subarray<A> const & a, long start_a, long end_a,
-			 subarray<B> const & b, long start_b, long end_b,
-			 std::vector<long> & edits,
-			 long edx1,
-			 long polarity,
-			 long p_lim)
+                         subarray<B> const & b, long start_b, long end_b,
+                         std::vector<long, QA(long)> & edits,
+                         long edx1,
+                         long polarity,
+                         long p_lim)
   {
 
     I(start_a >= 0);
@@ -386,69 +386,69 @@ struct jaffer_edit_calculator
     // easy case #1: B inserts only
     if (p_lim == 0)
       {
-	// A == B, no edits
-	if (len_a == len_b)
-	  return 0;
-	
-	long adx = start_a;
-	long bdx = start_b;
-	long edx0 = edx1;
-	
-	while (true)
-	  {	       
-	    if (bdx >= end_b)
-	      return len_b - len_a;
-	    
-	    if (adx >= end_a)
-	      {
-		for (long idx = bdx, edx = edx0; 
-		     idx < end_b; 
-		     ++idx, ++edx)
-		  edits[edx] = polarity * (idx+1);
-		
-		return len_b - len_a;		  
-	      }
-	    
-	    if (a[adx] == b[bdx])
-	      {
-		++adx; ++bdx;
-	      }
-	    else
-	      {
-		edits[edx0] = polarity * (bdx+1);
-		++bdx; ++edx0;
-	      }
-	  }
+        // A == B, no edits
+        if (len_a == len_b)
+          return 0;
+        
+        long adx = start_a;
+        long bdx = start_b;
+        long edx0 = edx1;
+        
+        while (true)
+          {               
+            if (bdx >= end_b)
+              return len_b - len_a;
+            
+            if (adx >= end_a)
+              {
+                for (long idx = bdx, edx = edx0; 
+                     idx < end_b; 
+                     ++idx, ++edx)
+                  edits[edx] = polarity * (idx+1);
+                
+                return len_b - len_a;                  
+              }
+            
+            if (a[adx] == b[bdx])
+              {
+                ++adx; ++bdx;
+              }
+            else
+              {
+                edits[edx0] = polarity * (bdx+1);
+                ++bdx; ++edx0;
+              }
+          }
       }
     
     // easy case #2: delete all A, insert all B
     else if (len_a <= p_lim) 
       {
-	I(len_a == p_lim);
+        I(len_a == p_lim);
 
-	long edx0 = edx1;
-	for (long idx = start_a; idx < end_a; ++idx, ++edx0)
-	  edits[edx0] = polarity * (-1 - idx);
-	
-	for (long jdx = start_b; jdx < end_b; ++jdx, ++edx0)
-	  edits[edx0] = polarity * (jdx + 1);
-	
-	return len_a + len_b;
+        long edx0 = edx1;
+        for (long idx = start_a; idx < end_a; ++idx, ++edx0)
+          edits[edx0] = polarity * (-1 - idx);
+        
+        for (long jdx = start_b; jdx < end_b; ++jdx, ++edx0)
+          edits[edx0] = polarity * (jdx + 1);
+        
+        return len_a + len_b;
       }
     
     // hard case: recurse on subproblems
     else
       {
-	return divide_and_conquer (a, start_a, end_a,
-				   b, start_b, end_b,
-				   edits, edx1, polarity, p_lim);
+        return divide_and_conquer (a, start_a, end_a,
+                                   b, start_b, end_b,
+                                   edits, edx1, polarity, p_lim);
       }
   }
   
   static void diff_to_edits(subarray<A> const & a, long m,
-			    subarray<B> const & b, long n,
-			    std::vector<long> & edits,
-			    long p_lim)
+                            subarray<B> const & b, long n,
+                            std::vector<long, QA(long)> & edits,
+                            long p_lim)
   {
     I(m <= n);
     cost_vec costs(m+n); // scratch array, ignored
@@ -457,32 +457,32 @@ struct jaffer_edit_calculator
     edits.clear();
     edits.resize(edit_distance, 0);
     long cost = diff_to_et(a, 0, m, 
-			   b, 0, n, 
-			   edits, 0, 1, (edit_distance - (n-m)) / 2);
+                           b, 0, n, 
+                           edits, 0, 1, (edit_distance - (n-m)) / 2);
     I(cost == edit_distance);
   }
   
-  static void edits_to_lcs (std::vector<long> const & edits, 
-			    subarray<A> const a, long m, long n,
-			    LCS output)
+  static void edits_to_lcs (std::vector<long, QA(long)> const & edits, 
+                            subarray<A> const a, long m, long n,
+                            LCS output)
   {
     long edx = 0, sdx = 0, adx = 0;
     typedef typename std::iterator_traits<A>::value_type vt;
-    std::vector<vt> lcs(((m + n) - edits.size()) / 2);
+    std::vector<vt, QA(vt)> lcs(((m + n) - edits.size()) / 2);
     while (true)
       {
-	long edit = (edx < static_cast<long>(edits.size())) ? edits[edx] : 0;
-	
-	if (adx >= m)
-	  break;
-	else if (edit > 0)
-	  { ++edx; }
-	else if (edit == 0)
-	  { lcs[sdx++] = a[adx++]; }
-	else if (adx >= (-1 - edit))
-	  { ++edx; ++adx; }
-	else
-	  { lcs[sdx++] = a[adx++]; }
+        long edit = (edx < static_cast<long>(edits.size())) ? edits[edx] : 0;
+        
+        if (adx >= m)
+          break;
+        else if (edit > 0)
+          { ++edx; }
+        else if (edit == 0)
+          { lcs[sdx++] = a[adx++]; }
+        else if (adx >= (-1 - edit))
+          { ++edx; ++adx; }
+        else
+          { lcs[sdx++] = a[adx++]; }
       }
 
     std::copy(lcs.begin(), lcs.end(), output);
@@ -491,13 +491,13 @@ struct jaffer_edit_calculator
 
 
 template <typename A,
-	  typename B,
-	  typename LCS>
+          typename B,
+          typename LCS>
 void _edit_script(A begin_a, A end_a,
-		  B begin_b, B end_b,
-		  long p_lim,
-		  std::vector<long> & edits_out,
-		  LCS ignored_out)
+                  B begin_b, B end_b,
+                  long p_lim,
+                  std::vector<long, QA(long)> & edits_out,
+                  LCS ignored_out)
 {
   typedef jaffer_edit_calculator<A,B,LCS> calc_t;
   long len_a = end_a - begin_a;
@@ -512,7 +512,7 @@ void _edit_script(A begin_a, A end_a,
       calc_t::diff_to_edits (b, len_b, a, len_a, edits, p_lim);
       calc_t::order_edits (edits, -1, ordered);
       for (size_t i = 0; i < ordered.size(); ++i)
-	ordered[i] *= -1;
+        ordered[i] *= -1;
     }
   else
     {
@@ -527,12 +527,12 @@ void _edit_script(A begin_a, A end_a,
 
 
 template <typename A,
-	  typename B,
-	  typename LCS>
+          typename B,
+          typename LCS>
 void _longest_common_subsequence(A begin_a, A end_a,
-				 B begin_b, B end_b,
-				 long p_lim,
-				 LCS out)
+                                 B begin_b, B end_b,
+                                 long p_lim,
+                                 LCS out)
 {
   typedef jaffer_edit_calculator<A,B,LCS> calc_t;
   long len_a = end_a - begin_a;
@@ -558,29 +558,29 @@ void _longest_common_subsequence(A begin_a, A end_a,
 
 
 void 
-longest_common_subsequence(std::vector<long>::const_iterator begin_a,
-			   std::vector<long>::const_iterator end_a,
-			   std::vector<long>::const_iterator begin_b,
-			   std::vector<long>::const_iterator end_b,
-			   long p_lim,
-			   std::back_insert_iterator< std::vector<long> > lcs)
+longest_common_subsequence(std::vector<long, QA(long)>::const_iterator begin_a,
+                           std::vector<long, QA(long)>::const_iterator end_a,
+                           std::vector<long, QA(long)>::const_iterator begin_b,
+                           std::vector<long, QA(long)>::const_iterator end_b,
+                           long p_lim,
+                           std::back_insert_iterator< std::vector<long, QA(long)> > lcs)
 {
   _longest_common_subsequence(begin_a, end_a,
-			      begin_b, end_b,
-			      p_lim, lcs);
+                              begin_b, end_b,
+                              p_lim, lcs);
 }
 
 void 
-edit_script(std::vector<long>::const_iterator begin_a,
-	    std::vector<long>::const_iterator end_a,
-	    std::vector<long>::const_iterator begin_b,
-	    std::vector<long>::const_iterator end_b,
-	    long p_lim,
-	    std::vector<long> & edits_out)
+edit_script(std::vector<long, QA(long)>::const_iterator begin_a,
+            std::vector<long, QA(long)>::const_iterator end_a,
+            std::vector<long, QA(long)>::const_iterator begin_b,
+            std::vector<long, QA(long)>::const_iterator end_b,
+            long p_lim,
+            std::vector<long, QA(long)> & edits_out)
 {
-  std::vector<long> lcs;
+  std::vector<long, QA(long)> lcs;
   _edit_script(begin_a, end_a, 
-	       begin_b, end_b, 
-	       p_lim, edits_out,
-	       back_inserter(lcs));
+               begin_b, end_b, 
+               p_lim, edits_out,
+               back_inserter(lcs));
 }
