@@ -35,6 +35,8 @@ extern "C" {
 #include "platform.hh"
 #include "transforms.hh"
 #include "paths.hh"
+#include "globish.hh"
+#include "basic_io.hh"
 
 // defined in {std,test}_hooks.lua, converted
 #include "test_hooks.h"
@@ -43,6 +45,14 @@ extern "C" {
 using namespace std;
 using boost::lexical_cast;
 
+// this lets the lua callbacks (monotone_*_for_lua) have access to the
+// app_state the're associated with.
+// it was added so that the confdir (normally ~/.monotone) can be specified on
+// the command line (and so known only to the app_state), and still be
+// available to lua
+// please *don't* use it for complex things that can throw errors
+static std::map<lua_State*, app_state*> map_of_lua_to_app;
+
 static int panic_thrower(lua_State * st)
 {
   throw oops("lua panic");
@@ -111,8 +121,9 @@ Lua
   }
 
   bool ok() 
-  { 
-    L(F("Lua::ok(): failed = %i") % failed);
+  {
+    if (failed) 
+      L(F("Lua::ok(): failed"));
     return !failed; 
   }
 
@@ -605,7 +616,44 @@ extern "C"
     const char *str = lua_tostring(L, -1);
     boost::cmatch what;
 
-    lua_pushboolean(L, boost::regex_search(str, what, boost::regex(re)));
+    bool result = false;
+    try {
+      result = boost::regex_search(str, what, boost::regex(re));
+    } catch (boost::bad_pattern e) {
+      lua_pushstring(L, e.what());
+      lua_error(L);
+      return 0;
+    }
+    lua_pushboolean(L, result);
+    return 1;
+  }
+
+  static int
+  monotone_globish_match_for_lua(lua_State *L)
+  {
+    const char *re = lua_tostring(L, -2);
+    const char *str = lua_tostring(L, -1);
+
+    bool result = false;
+    try {
+      string r(re);
+      string n;
+      string s(str);
+      result = globish_matcher(r, n)(s);
+    } catch (informative_failure & e) {
+      lua_pushstring(L, e.what.c_str());
+      lua_error(L);
+      return 0;
+    } catch (boost::bad_pattern & e) {
+      lua_pushstring(L, e.what());
+      lua_error(L);
+      return 0;
+    } catch (...) {
+      lua_pushstring(L, "Unknown error.");
+      lua_error(L);
+      return 0;
+    }
+    lua_pushboolean(L, result);
     return 1;
   }
 
@@ -616,6 +664,83 @@ extern "C"
     lua_pushstring(L, gettext(msgid));
     return 1;
   }
+
+  static int
+  monotone_get_confdir_for_lua(lua_State *L)
+  {
+    map<lua_State*, app_state*>::iterator i = map_of_lua_to_app.find(L);
+    if (i != map_of_lua_to_app.end())
+      {
+        system_path dir = i->second->get_confdir();
+        string confdir = dir.as_external();
+        lua_pushstring(L, confdir.c_str());
+      }
+    else
+      lua_pushnil(L);
+    return 1;
+  }
+
+  static int
+  monotone_parse_basic_io_for_lua(lua_State *L)
+  {
+    vector<pair<string, vector<string> > > res;
+    const char *str = lua_tostring(L, -1);
+    std::istringstream iss(str);
+    basic_io::input_source in(iss, "monotone_parse_basic_io_for_lua");
+    basic_io::tokenizer tok(in);
+    try
+      {
+        string got;
+        basic_io::token_type tt;
+        do
+          {
+            tt = tok.get_token(got);
+            switch (tt)
+              {
+              case basic_io::TOK_SYMBOL:
+                res.push_back(make_pair(got, vector<string>()));
+                break;
+              case basic_io::TOK_STRING:
+              case basic_io::TOK_HEX:
+                E(!res.empty(), boost::format("bad input to parse_basic_io"));
+                res.back().second.push_back(got);
+                break;
+              default:
+                break;
+              }
+          }
+        while (tt != basic_io::TOK_NONE);
+      }
+    catch (informative_failure & e)
+      {// there was a syntax error in our string
+        lua_pushnil(L);
+        return 0;
+      }
+    lua_newtable(L);
+    int n = 1;
+    for (vector<pair<string, vector<string> > >::const_iterator i = res.begin();
+         i != res.end(); ++i)
+      {
+        lua_pushnumber(L, n++);
+        lua_newtable(L);
+        lua_pushstring(L, "name");
+        lua_pushstring(L, i->first.c_str());
+        lua_settable(L, -3);
+        lua_pushstring(L, "values");
+        lua_newtable(L);
+        int m = 1;
+        for (vector<string>::const_iterator j = i->second.begin();
+             j != i->second.end(); ++j)
+          {
+            lua_pushnumber(L, m++);
+            lua_pushstring(L, j->c_str());
+            lua_settable(L, -3);
+          }
+        lua_settable(L, -3);
+        lua_settable(L, -3);
+      }
+    return 1;
+  }
 }
 
 
@@ -646,6 +771,8 @@ lua_hooks::lua_hooks()
   lua_register(st, "include", monotone_include_for_lua);
   lua_register(st, "includedir", monotone_includedir_for_lua);
   lua_register(st, "gettext", monotone_gettext_for_lua);
+  lua_register(st, "get_confdir", monotone_get_confdir_for_lua);
+  lua_register(st, "parse_basic_io", monotone_parse_basic_io_for_lua);
 
   // add regex functions:
   lua_newtable(st);
@@ -657,13 +784,32 @@ lua_hooks::lua_hooks()
   lua_pushcfunction(st, monotone_regex_search_for_lua);
   lua_settable(st, -3);
 
+  // add globish functions:
+  lua_newtable(st);
+  lua_pushstring(st, "globish");
+  lua_pushvalue(st, -2);
+  lua_settable(st, LUA_GLOBALSINDEX);
+
+  lua_pushstring(st, "match");
+  lua_pushcfunction(st, monotone_globish_match_for_lua);
+  lua_settable(st, -3);
+
   lua_pop(st, 1);
 }
 
 lua_hooks::~lua_hooks()
 {
+  map<lua_State*, app_state*>::iterator i = map_of_lua_to_app.find(st);
   if (st)
     lua_close (st);
+  if (i != map_of_lua_to_app.end())
+    map_of_lua_to_app.erase(i);
+}
+
+void
+lua_hooks::set_app(app_state *_app)
+{
+  map_of_lua_to_app.insert(make_pair(st, _app));
 }
 
 static bool 
@@ -708,7 +854,9 @@ lua_hooks::add_std_hooks()
 void 
 lua_hooks::default_rcfilename(system_path & file)
 {
-  file = system_path(get_homedir()) / ".monotone/monotonerc";
+  map<lua_State*, app_state*>::iterator i = map_of_lua_to_app.find(st);
+  I(i != map_of_lua_to_app.end());
+  file = i->second->get_confdir() / "monotonerc";
 }
 
 void 
@@ -842,8 +990,8 @@ lua_hooks::hook_get_branch_key(cert_value const & branchname,
 }
 
 bool 
-lua_hooks::hook_get_priv_key(rsa_keypair_id const & k,
-                             base64< arc4<rsa_priv_key> > & priv_key )
+lua_hooks::hook_get_key_pair(rsa_keypair_id const & k,
+                             keypair & kp)
 {
   string key;
   bool ok = Lua(st)
@@ -853,7 +1001,11 @@ lua_hooks::hook_get_priv_key(rsa_keypair_id const & k,
     .extract_str(key)
     .ok();
 
-  priv_key = key;
+  size_t pos = key.find("#");
+  if (pos == std::string::npos)
+    return false;
+  kp.pub = key.substr(0, pos);
+  kp.priv = key.substr(pos+1);
   return ok;
 }
 
