@@ -13,10 +13,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <boost/filesystem/path.hpp>
 
-#include "cryptopp/osrng.h"
 #include "file_io.hh"
+#include "botan/botan.h"
 
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -29,12 +28,10 @@ monotone_mkstemp(std::string &tmpl)
   int i = 0;
   int count = 0, fd = -1;
   std::string tmp;
-  fs::path path;
 
   static const char letters[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
   static const int NLETTERS = sizeof (letters) - 1;
-  static CryptoPP::AutoSeededRandomPool mkstemp_urandom;
 
   len = tmpl.length();
   if (len < 6 || tmpl.rfind("XXXXXX") != len-6)
@@ -45,17 +42,15 @@ monotone_mkstemp(std::string &tmpl)
       tmp = tmpl.substr(0, len-6);
 
       for (i = 0; i < 6; ++i)
-        tmp.append(1, letters[mkstemp_urandom.GenerateByte() % NLETTERS]);
+        tmp.append(1, letters[Botan::Global_RNG::random(Botan::Nonce) % NLETTERS]);
       fd = open(tmp.c_str(), O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);      
       if (fd >= 0)
       {
-	fs::path path;
-	path = mkpath(tmp);
-	tmpl = path.native_directory_string();
-	return fd;
+        tmpl = tmp;
+        return fd;
       }
       else if (errno != EEXIST)
-	break;
+        break;
     }  
   return -1;
 }
