@@ -1590,8 +1590,7 @@ build_changesets_from_existing_revs(app_state & app)
   {
     // early short-circuit to avoid failure after lots of work
     rsa_keypair_id key;
-    N(guess_default_key(key,app),
-      F("no unique private key for cert construction"));
+    get_user_key(key,app);
     require_password(key, app);
   }
 
@@ -1622,8 +1621,7 @@ build_changesets_from_manifest_ancestry(app_state & app)
   {
     // early short-circuit to avoid failure after lots of work
     rsa_keypair_id key;
-    N(guess_default_key(key,app),
-      F("no unique private key for cert construction"));
+    get_user_key(key,app);
     require_password(key, app);
   }
 
