@@ -30,7 +30,8 @@ using boost::format;
 sanity global_sanity;
 
 sanity::sanity() : 
-  debug(false), quiet(false), relaxed(false), logbuf(0xffff), already_dumping(false)
+  debug(false), quiet(false), relaxed(false), logbuf(0xffff), 
+  already_dumping(false), clean_shutdown(false)
 {
   std::string flavour;
   get_system_flavour(flavour);
@@ -179,6 +180,7 @@ sanity::naughty_failure(string const & expr, format const & explain,
   log(format("%s:%d: usage constraint '%s' violated\n") % file % line % expr,
       file.c_str(), line);
   prefix_lines_with(_("misuse: "), explain.str(), message);
+  gasp();
   throw informative_failure(message);
 }
 
