@@ -88,7 +88,16 @@ function ignore_file(name)
    end
    for i, line in pairs(ignored_files)
    do
-      if (regex.search(line, name)) then return true end
+      local pcallstatus, result = pcall(function() if (regex.search(line, name)) then return true else return false end end)
+      if pcallstatus == true then
+          -- no error from the regex.search call
+          if result == true then return true end
+      else
+          -- regex.search had a problem, warn the user their .mt-ignore file syntax is wrong
+          io.stderr:write("WARNING: the line '" .. line .. "' in your .mt-ignore file caused exception '" .. result .. "'"
+                           .. " while matching filename '" .. name .. "', ignoring this regex for all remaining files.\n")
+          table.remove(ignored_files, i)
+      end
    end
    -- c/c++
    if (string.find(name, "%.a$")) then return true end
@@ -98,6 +107,7 @@ function ignore_file(name)
    if (string.find(name, "%.lo$")) then return true end
    if (string.find(name, "^core$")) then return true end
    if (string.find(name, "/core$")) then return true end
+   if (string.find(name, "/core%.%d+$")) then return true end
    -- python
    if (string.find(name, "%.pyc$")) then return true end
    if (string.find(name, "%.pyo$")) then return true end
@@ -184,7 +194,11 @@ function edit_comment(basetext, user_log_message)
    local tmp, tname = temp_file()
    if (tmp == nil) then return nil end
    basetext = "MT: " .. string.gsub(basetext, "\n", "\nMT: ") .. "\n"
-   tmp:write(user_log_message)
+   if user_log_message == "" then
+      tmp:write("\n")
+   else
+      tmp:write(user_log_message)
+   end
    tmp:write(basetext)
    io.close(tmp)
 
@@ -220,6 +234,12 @@ function persist_phrase_ok()
    return true
 end
 
+
+function use_inodeprints()
+   return false
+end
+
+
 -- trust evaluation hooks
 
 function intersection(a,b)
@@ -336,18 +356,17 @@ function merge2_emacs_cmd(emacs, lfile, rfile, outfile)
    local elisp = "(ediff-merge-files \"%s\" \"%s\" nil \"%s\")"
    return 
    function()
-      return execute(emacs, "-eval", 
+      return execute(emacs, "--eval", 
                      string.format(elisp, lfile, rfile, outfile))
    end
 end
 
 function merge3_emacs_cmd(emacs, lfile, afile, rfile, outfile)
    local elisp = "(ediff-merge-files-with-ancestor \"%s\" \"%s\" \"%s\" nil \"%s\")"
-   local cmd_fmt = "%s -eval " .. elisp
    return 
    function()
-      execute(emacs, "-eval", 
-              string.format(elisp, lfile, rfile, afile, outfile))
+      return execute(emacs, "--eval", 
+                     string.format(elisp, lfile, rfile, afile, outfile))
    end
 end
 
@@ -591,7 +610,7 @@ function get_preferred_merge3_command (tbl)
          cmd = merge3_emacs_cmd ("emacs", lfile, afile, rfile, outfile) 
       end
    elseif string.find(editor, "vim") ~= nil then
-      io.write (string.format("\nWARNING: 'vim' was choosen to perform external 2-way merge.\n"..
+      io.write (string.format("\nWARNING: 'vim' was choosen to perform external 3-way merge.\n"..
           "You should merge all changes to *LEFT* file due to limitation of program\n"..
           "arguments.  The order of the files is ancestor, left, right.\n\n")) 
       if os.getenv ("DISPLAY") ~= nil and program_exists_in_path ("gvim") then 
@@ -754,9 +773,6 @@ function expand_date(str)
    return nil
 end
 
-function use_inodeprints()
-   return false
-end
 
 external_diff_default_args = "-u"
 
@@ -772,3 +788,73 @@ function external_diff(file_path, data_old, data_new, is_binary, diff_args, rev_
    os.remove (new_file);
 end
 
+-- netsync permissions hooks (and helper)
+
+function globish_match(glob, str)
+      local pcallstatus, result = pcall(function() if (globish.match(glob, str)) then return true else return false end end)
+      if pcallstatus == true then
+          -- no error
+          return result
+      else
+          -- globish.match had a problem
+          return nil
+      end
+end
+
+function get_netsync_read_permitted(branch, ident)
+   local permfile = io.open(get_confdir() .. "/read-permissions", "r")
+   if (permfile == nil) then return false end
+   local dat = permfile:read("*a")
+   local res = parse_basic_io(dat)
+   if res == nil then
+      io.stderr:write("file read-permissions cannot be parsed\n")
+      return false
+   end
+   local matches = false
+   local cont = false
+   for i, item in pairs(res)
+   do
+      -- legal names: pattern, allow, deny, continue
+      if item.name == "pattern" then
+         if matches and not cont then return false end
+         matches = false
+         for j, val in pairs(item.values) do
+            if globish_match(val, branch) then matches = true end
+         end
+      elseif item.name == "allow" then if matches then
+         for j, val in pairs(item.values) do
+            if val == "*" then return true end
+            if globish_match(val, ident) then return true end
+         end
+      end elseif item.name == "deny" then if matches then
+         for j, val in pairs(item.values) do
+            if globish_match(val, ident) then return false end
+         end
+      end elseif item.name == "continue" then if matches then
+         cont = true
+         for j, val in pairs(item.values) do
+            if val == "false" or val == "no" then cont = false end
+         end
+      end elseif item.name ~= "comment" then
+         io.stderr:write("unknown symbol in read-permissions: " .. item.name .. "\n")
+         return false
+      end
+   end
+   return false
+end
+
+function get_netsync_write_permitted(ident)
+   local permfile = io.open(get_confdir() .. "/write-permissions", "r")
+   if (permfile == nil) then
+      return false
+   end
+   local line = permfile:read()
+   while (line ~= nil) do
+      local _, _, ln = string.find(line, "%s*([^%s]*)%s*")
+      if ln == "*" then return true end
+      if globish_match(ln, ident) then return true end
+      line = permfile:read()
+   end
+   io.close(permfile)
+   return false
+end
